//
//  PathRouter.m
//  TouristHelper
//
//  Adam Eberbach

#import "PathRouter.h"
#import <MapKit/MapKit.h>
#import <PulseSDK/PulseSDK.h>

@interface PathRouter ()
@property (nonatomic, strong) NSMutableArray *workingCopy;
@end

@implementation PathRouter

// given an array of objects conforming to <MKAnnotation>, order them in a reasonably short path.

- (void)orderPathPoints:(NSArray *)points
{
  [PulseSDK labelTouch:@"Calculating Path"];
    
  dispatch_queue_t backgroundQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0);
  dispatch_async(backgroundQueue, ^{
    
    // build the set of locations to order
    NSMutableArray *locations = [NSMutableArray arrayWithCapacity:[points count]];
    for(id<MKAnnotation> annotationObject in points) {
      // these need to be in CLLocation objects because that object provides the distance from location method
      [locations addObject:[[CLLocation alloc] initWithLatitude:annotationObject.coordinate.latitude longitude:annotationObject.coordinate.longitude]];
    }
    
    NSMutableArray *bestRoute;
    CLLocationDistance shortestRouteDistance = CLLocationDistanceMax;
    
    // fresh copy of working data
    self.workingCopy = [locations mutableCopy];
    
    // starting at location 0, sort the locations in the location to visit
    NSRange sortRange = NSMakeRange(1, [self.workingCopy count] - 1);
    [self orderLocationsInRange:sortRange byDistanceFromLocation:[self.workingCopy objectAtIndex:0]];
    
    // check the total distance of the route just calculated
    CLLocationDistance thisRouteDistance = 0;
    CLLocation *previous = self.workingCopy[0];
    for(CLLocation *location in self.workingCopy) {
      // checking the first location against itself doesn't matter - a distance of zero makes no difference!
      thisRouteDistance += [location distanceFromLocation:previous];
      previous = location;
    }
    
    if(thisRouteDistance < shortestRouteDistance) {
      NSLog(@"choosing shorter route %f", thisRouteDistance);
      
      shortestRouteDistance = thisRouteDistance;
      bestRoute = [self.workingCopy mutableCopy];
    }
    
    // calculation of the path to all the sights, without blocking the main (UI) thread
    
    dispatch_async(dispatch_get_main_queue(), ^{
      // inform delegate of path completion
      [self.delegate pathRouter:self didCompleteWithOrderedPoints:bestRoute];
    });
  });
}

// a very naive route calculation, because travelling salesman is hard - instead of actually finding the next
// shortest point, just randomly order the points. FIXME!

- (void)orderLocationsInRange:(NSRange)range byDistanceFromLocation:(CLLocation *)currentLocation {
  
  if(range.length == 1) {
    // termination condition
    return;
  }
  else
  {
    // remaining coordinates in range must be sorted by proximity to currentCoordinate
    NSArray *sortedRemainingLocations = [[self.workingCopy subarrayWithRange:range] sortedArrayUsingComparator:^(id location1, id location2) {
      
      CLLocationDistance distance1 = [location1 distanceFromLocation:currentLocation];
      CLLocationDistance distance2 = [location2 distanceFromLocation:currentLocation];
      
      if (distance1 > distance2) {
        return NSOrderedDescending;
      } else if (distance2 < distance1) {
        return NSOrderedAscending;
      } else {
        return NSOrderedSame;
      }
    }];
    
    // replace the objects in locations in range with the sorted objects
    [self.workingCopy replaceObjectsInRange:range withObjectsFromArray:sortedRemainingLocations];
    
    // recurse to order the rest of the coordinates
    [self orderLocationsInRange:NSMakeRange(range.location + 1, range.length - 1) byDistanceFromLocation:[sortedRemainingLocations objectAtIndex:0]];
  }
}

@end
