//
//  SightFinder.m
//  TouristHelper
//
//  Adam Eberbach

#import "SightFinder.h"
#import "Sight.h"
#import "AFHTTPClient.h"
#import "AFJSONRequestOperation.h"
#import "FlickrServices.h"

@interface SightFinder ()
@property (nonatomic, strong) AFHTTPClient *networkClient;
@end

@implementation SightFinder

- (id)init {
  
  self = [super init];
  if(self) {
    
    NSURL *flickrBaseURL = [NSURL URLWithString:kFlickrBaseURL];
    self.networkClient = [AFHTTPClient clientWithBaseURL:flickrBaseURL];
    
    // AFNetworking: https://github.com/AFNetworking/AFNetworking/issues/614 - added includes to .pch
    [self.networkClient setReachabilityStatusChangeBlock:^(AFNetworkReachabilityStatus status) {
      if (status == AFNetworkReachabilityStatusNotReachable) {
        // Not reachable
      } else {
        // Reachable
      }
      
      if (status == AFNetworkReachabilityStatusReachableViaWiFi) {
        // On wifi
      }
    }];
  }
  return self;
}

// kick off a request for count sights matching tag
- (void)findSightsCount:(NSUInteger)count forTag:(NSString *)tag aroundLocation:(CLLocation *)location withRadius:(CGFloat)kilometres  {
  
  NSString *latitude = [NSString stringWithFormat:@"%f", location.coordinate.latitude];
  NSString *longitude = [NSString stringWithFormat:@"%f", location.coordinate.longitude];
  NSString *radius = [NSString stringWithFormat:@"%f", kilometres];
  NSString *path = [[NSString alloc] initWithFormat:@"%@%@", kFlickrQueryPath, kFlickrMethodPhotosSearch];
  NSString *resultsPerPage = [NSString stringWithFormat:@"%d", (int)count];
  
  NSDictionary *params = @{kFlickrParameterAPIKey : kFlickrGeoPhotoAPIKey,
                           kFlickrParameterTags : tag,
                           kFlickrParameterLat : latitude,
                           kFlickrParameterLon : longitude,
                           kFlickrParameterRadius : radius,
                           kFlickrParameterFormat : kFlickrParameterFormatValue,
                           kFlickrParameterHasGeo : kFlickrParameterHasGeoValue,
                           kFlickrParameterExtras : kFlickrParameterExtrasValue,
                           kFlickrParameterVanillaJSON : kFlickrParameterVanillaJSONValue,
                           kFlickrParameterContentType : kFlickrParameterContentTypeValue,
                           kFlickrParameterResultsPerPage : resultsPerPage,
                           kFlickrParameterMediaType : kFlickrParameterMediaTypeValue,
                           };
  
  
  NSMutableURLRequest *request = [_networkClient requestWithMethod:@"GET" path:path parameters:params];
  
  AFJSONRequestOperation *operation =
  [AFJSONRequestOperation JSONRequestOperationWithRequest:request
                                                  success:^(NSURLRequest *request, NSHTTPURLResponse *response, id JSON) {
                                                    
                                                    [self processIncomingSightJSON:JSON];
                                                    
                                                    [_delegate SightFinder:self didCompleteRequestWithStatus:kSightFinderSearchSuccess];
                                                    
                                                  } failure:^(NSURLRequest *request, NSHTTPURLResponse *response, NSError *error, id JSON) {
                                                    
                                                    [_delegate SightFinder:self didCompleteRequestWithStatus:kSightFinderSearchFailure];
                                                    
                                                    NSLog(@"Error %@", [error localizedDescription]);
                                                  }];
  [operation start];
  
}

- (void)processIncomingSightJSON:(NSDictionary *)jsonDict {
  
  NSDictionary *sightsDictionary = jsonDict[@"photos"];
  NSArray *sightsArray = sightsDictionary[@"photo"];
  NSMutableArray *processedSights = [NSMutableArray array];
  
  for(NSDictionary *sightDictionary in sightsArray) {
    
    NSString *title = sightDictionary[@"title"];
    NSNumber *latitudeNumber = sightDictionary[@"latitude"];
    NSNumber *longitudeNumber = sightDictionary[@"longitude"];
    CLLocation *location = [[CLLocation alloc] initWithLatitude:[latitudeNumber doubleValue]
                                                      longitude:[longitudeNumber doubleValue]];
    // construct the base URL for the image - adding the suffix and .jpg gets the image URL itself
    NSString *baseImageURLString = [NSString stringWithFormat:@"http://farm%@.staticflickr.com/%@/%@_%@",
                                    sightDictionary[@"farm"],
                                    sightDictionary[@"server"],
                                    sightDictionary[@"id"],
                                    sightDictionary[@"secret"]];
    
    Sight *newSight = [[Sight alloc] initWithTitle:title
                                        coordinate:location.coordinate
                                           baseURL:baseImageURLString];
    [processedSights addObject:newSight];
  }
  
  // the visible property sights contains all the results to be used
  self.sights = [NSArray arrayWithArray:processedSights];
}


@end
