//
//  MapSightsViewController.m
//  TouristHelper
//
//  Adam Eberbach

#import "MapSightsViewController.h"
#import "BigImageViewController.h"
#import "PathRouter.h"
#import <MapKit/MapKit.h>
#import <PulseSDK/PulseSDK.h>

static const NSUInteger kMapSightsCount = (20);

// MapKit distances are measured in meters
static const CLLocationDistance kMapDefaultLatitudinalMeters = (20000.0);
static const CLLocationDistance kMapDefaultLongitudinalMeters = (20000.0);
static const CGFloat kAccessoryThumbnailMaxDimension = (24.0);
static const CGFloat kMapEnclosingRectEdgeInsets = (10.0);

@interface MapSightsViewController ()
@property (nonatomic, strong) CLLocationManager *locationManager;
@property (nonatomic, strong) SightFinder *sightFinder;
@property (nonatomic) NSUInteger activityCount;
@end

@implementation MapSightsViewController

- (void)viewDidLoad
{
  [super viewDidLoad];
  
  self.sightFinder = [[SightFinder alloc] init];
  _sightFinder.delegate = self;
  
  self.locationManager = [[CLLocationManager alloc] init];
  _locationManager.desiredAccuracy = kCLLocationAccuracyBest;
  _locationManager.delegate = self;
  [_locationManager startUpdatingLocation];
  
  self.activityCount = 0;
}

- (IBAction)refreshLocation:(UIBarButtonItem *)sender {
  [_locationManager startUpdatingLocation];
}

#pragma mark - CLLocationManagerDelegate

- (void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray *)locations {
  
  [_locationManager stopUpdatingLocation];
  
  CLLocation *location = [locations objectAtIndex:0];
  
  // centre map on default span for location
  MKCoordinateRegion displayRegion = MKCoordinateRegionMakeWithDistance(location.coordinate,
                                                                        kMapDefaultLatitudinalMeters,
                                                                        kMapDefaultLongitudinalMeters);
  [self.mapView setRegion:displayRegion animated:YES];
  
  // kick off fetch of nearby images
  [_sightFinder findSightsCount:kMapSightsCount
                         forTag:@"beer"
                 aroundLocation:location
                     withRadius:kMapDefaultLatitudinalMeters / 2 / 1000];
}

- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
  
  UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Location Unavailable", nil)
                                                      message:NSLocalizedString(@"Cannot determine device location.", nil)
                                                     delegate:nil
                                            cancelButtonTitle:NSLocalizedString(@"Cancel", nil)
                                            otherButtonTitles:nil];
  [alertView show];
}

#pragma - MKMapViewDelegate

- (MKAnnotationView *)mapView:(MKMapView *)mapview viewForAnnotation:(id <MKAnnotation>)annotation
{
  [PulseSDK labelTouch:@"Providing Annotation View"];
    
  if ([annotation isKindOfClass:[MKUserLocation class]]) {
    return nil;
  }
  
  static NSString* kSightAnnotationIdentifier = @"kSightAnnotationIdentifier";
  MKAnnotationView *sightAnnotationView;
  
  sightAnnotationView = [mapview dequeueReusableAnnotationViewWithIdentifier:kSightAnnotationIdentifier];
  if(sightAnnotationView == nil) {
    sightAnnotationView = [[MKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:kSightAnnotationIdentifier];
    sightAnnotationView.canShowCallout = YES;
  }
  
  Sight *sight = (Sight *)annotation;
  UIImage *thumbnailImage = [sight thumbnailImage]; // causes a load and refresh of thumbnail...
  if(thumbnailImage == nil) {
    thumbnailImage = [UIImage imageNamed:@"DefaultLocationIcon.png"];
  }
  
  UIImageView *leftAccessoryView = [[UIImageView alloc] initWithImage:thumbnailImage];
  CGRect leftAccessoryViewFrame = leftAccessoryView.frame;
  leftAccessoryViewFrame.size.width = MIN(leftAccessoryView.frame.size.width, kAccessoryThumbnailMaxDimension);
  leftAccessoryViewFrame.size.height = MIN(leftAccessoryView.frame.size.height, kAccessoryThumbnailMaxDimension);
  leftAccessoryView.frame = leftAccessoryViewFrame;
  sightAnnotationView.leftCalloutAccessoryView = leftAccessoryView;
  
  UIButton *displayImageButton = [UIButton buttonWithType:UIButtonTypeDetailDisclosure];
  [displayImageButton addTarget:sight action:@selector(initiateImageDisplay) forControlEvents:UIControlEventTouchUpInside];
  sightAnnotationView.rightCalloutAccessoryView = displayImageButton;
  
  return sightAnnotationView;
}

// Set appearance of the lines on the map between points

-(MKOverlayView *)mapView:(MKMapView *)mapView viewForOverlay:(id<MKOverlay>)overlay{
  
  if ([overlay isKindOfClass:[MKPolyline class]])
  {
    MKPolylineView *routeView = [[MKPolylineView alloc] initWithPolyline:overlay];
    routeView.strokeColor = [UIColor blackColor];
    routeView.lineWidth = 1.0;
    return routeView;
  }
  return nil;
}

#pragma mark - SightFinderDelegate

- (void)SightFinder:(SightFinder *)sightFinder didCompleteRequestWithStatus:(SightFinderStatus)status {
  
  switch(status) {
    case kSightFinderSearchFailure:
    {
      UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Request Failed", nil)
                                                          message:NSLocalizedString(@"Could not find any interesting sights nearby.", nil)
                                                         delegate:nil
                                                cancelButtonTitle:NSLocalizedString(@"Cancel", nil)
                                                otherButtonTitles:nil];
      [alertView show];
    }
      break;
      
    case kSightFinderSearchSuccess:
    {
      // great, new points to plot - hand them off to PathRouter to put them in a visiting order
      PathRouter *pathRouter = [[PathRouter alloc] init];
      
      [PulseSDK instrumentClass:[PathRouter class] selector:@selector(orderPathPoints:)];
      
      pathRouter.delegate = self;
      [self taskStarting];
      
      // the path calculation is handed off to pathRouter - wait for notification by delegate method for it to finish
      [pathRouter orderPathPoints:sightFinder.sights];
    }
      break;
  }
}

#pragma mark - PathRouterDelegate

- (void)pathRouter:(PathRouter *)pathRouter didCompleteWithOrderedPoints:(NSArray *)orderedPoints
{
  [self taskEnded];
  
  // turn this back into an array of CLLocationCoordinates for easy polyline
  NSMutableArray *locationCoordinates = [NSMutableArray array];
  for(CLLocation *location in orderedPoints) {
    [locationCoordinates addObject:[NSValue valueWithMKCoordinate:location.coordinate]];
  }
  
  NSInteger numberOfCoordinates = [locationCoordinates count];
  CLLocationCoordinate2D coordinates[numberOfCoordinates];
  
  [self.mapView removeOverlays:[self.mapView overlays]];
  int i = 0;
  for(NSValue *coordinateValue in locationCoordinates) {
    coordinates[i] = [coordinateValue MKCoordinateValue];
    i++;
  }
  MKPolyline *polyLine = [MKPolyline polylineWithCoordinates:coordinates count:numberOfCoordinates];
  [self.mapView addOverlay:polyLine];
  
  // remove all existing annotations
  [self.mapView removeAnnotations:[self.mapView annotations]];
  
  // add a new annotation for each sight
  for(Sight *sight in self.sightFinder.sights) {

    // the sight delegate method updates the thumbnail on each annotation as an image is fetched
    sight.delegate = self;
    [self.mapView addAnnotation:sight];
  }
  
  // adjust map span to include all annotations
  MKMapRect zoomRect = MKMapRectNull;
  for (id <MKAnnotation> annotation in self.mapView.annotations) {
    
    MKMapPoint annotationPoint = MKMapPointForCoordinate(annotation.coordinate);
    MKMapRect pointRect = MKMapRectMake(annotationPoint.x, annotationPoint.y, 0.1, 0.1);
    if (MKMapRectIsNull(zoomRect)) {
      zoomRect = pointRect;
    }else{
      // the rect grows to include each point
      zoomRect = MKMapRectUnion(zoomRect, pointRect);
    }
  }
  [self.mapView setVisibleMapRect:zoomRect edgePadding:UIEdgeInsetsMake(kMapEnclosingRectEdgeInsets,
                                                                    kMapEnclosingRectEdgeInsets,
                                                                    kMapEnclosingRectEdgeInsets,
                                                                    kMapEnclosingRectEdgeInsets) animated:YES];
  
}

#pragma mark - Activity Indicator

- (void)taskStarting {
  
  self.activityCount++;
  [self.activityIndicator startAnimating];
}

- (void)taskEnded {
  
  if(--self.activityCount == 0) {
    [self.activityIndicator stopAnimating];
  }
}

#pragma mark - SightDelegate

- (void)sightBeginningLargeImageDownload {
  
  [self taskStarting];
}

- (void)sightsDownloadEnded {
  
  [self taskEnded];
}

- (void)sightDisplayImage:(Sight *)sight {
  
  BigImageViewController *bigImageViewController = [[BigImageViewController alloc] init];
  bigImageViewController.fullImage = sight.fullImage;
  [self.navigationController pushViewController:bigImageViewController animated:YES];
  [self.activityIndicator stopAnimating];
}

- (void)sightDidUpdateAvailableThumbnail:(Sight *)sight {
  
  [self.mapView removeAnnotation:sight];
  [self.mapView addAnnotation:sight];
}

@end
